# -*- coding: utf-8 -*-
"""Viewlets."""
import json

from plone import api
from plone.app.layout.viewlets.common import ViewletBase

from collective.querynextprev import QUERY, PREVIOUS_UIDS, NEXT_UIDS
from collective.querynextprev.interfaces import INextPrevNotNavigable
from collective.querynextprev.utils import (
    first_common_item, get_next_items, get_previous_items, expire_session_data,
    convert_to_str)


class NextPrevNavigationViewlet(ViewletBase):  # noqa #pylint: disable=W0223

    """Navigation viewlet for next/previous."""

    is_navigable = False
    previous_uids = []
    next_uids = []

    def update(self):
        if INextPrevNotNavigable.providedBy(self.context):
            return

        session = self.request.SESSION
        if session.has_key(QUERY):  # noqa
            query = session[QUERY]
            params = convert_to_str(json.loads(query))
            catalog = api.portal.get_tool('portal_catalog')
            brains = catalog.searchResults(**params)

            # if we have too many results, we return and delete session data
            max_res = api.portal.get_registry_record('collective.querynextprev.maxresults') or 250
            if len(brains) > max_res:
                self.is_navigable = False
                expire_session_data(self.request)
                return

            uids = [brain.UID for brain in brains]  # noqa #pylint: disable=E1103
            context_uid = self.context.UID()
            if context_uid in uids and len(uids) > 1:
                self.is_navigable = True
                context_index = uids.index(context_uid)
                self.previous_uids = list(reversed(
                    get_previous_items(uids, context_index)))
                self.next_uids = get_next_items(uids, context_index)
                session[PREVIOUS_UIDS] = json.dumps(self.previous_uids)
                session[NEXT_UIDS] = json.dumps(self.next_uids)
                return  # don't delete session data
            elif session.has_key(PREVIOUS_UIDS) or session.has_key(NEXT_UIDS):  # noqa #pylint: disable=C0301
                # context is not in results anymore
                # get previous
                old_previous = json.loads(session[PREVIOUS_UIDS])
                previous_item = first_common_item(uids, old_previous)
                if previous_item:
                    self.is_navigable = True
                    index = uids.index(previous_item)
                    self.previous_uids = list(reversed(
                        get_previous_items(uids, index, include_index=True)))
                    session[PREVIOUS_UIDS] = json.dumps(self.previous_uids)

                # get next
                old_next = json.loads(session[NEXT_UIDS])
                next_item = first_common_item(uids, old_next)
                if next_item:
                    self.is_navigable = True
                    index = uids.index(next_item)
                    self.next_uids = get_next_items(
                        uids, index, include_index=True)
                    session[NEXT_UIDS] = json.dumps(self.next_uids)

                if previous_item or next_item:
                    return  # don't delete session data

            expire_session_data(self.request)



# -*- coding: utf-8 -*-
"""Views."""
import json

from Products.Five.browser import BrowserView
from plone import api

from collective.querynextprev import QUERY, SEARCH_URL, NEXT_UIDS, PREVIOUS_UIDS  # noqa #pylint: disable=C0301
from collective.querynextprev.utils import (
    expire_session_data, first_common_item, get_previous_items, get_next_items,
    convert_to_str)


class GoToNextItem(BrowserView):

    """Redirect to next item in query."""

    uids_param = NEXT_UIDS

    def get_uids(self):
        """Get uids of the query results."""
        catalog = api.portal.get_tool('portal_catalog')
        params = convert_to_str(json.loads(self.request.SESSION[QUERY]))
        return [brain.UID for brain in catalog.searchResults(**params)]  # noqa #pylint: disable=E1103

    def __call__(self):
        request = self.request
        session = request.SESSION
        if session.has_key(QUERY) and session.has_key(self.uids_param):  # noqa
            next_uids = convert_to_str(json.loads(session[self.uids_param]))

            # reexecute the query to search within most recent results
            new_uids = self.get_uids()

            # search UID starting from context index in uids
            uid = first_common_item(next_uids, new_uids)
            if uid is not None:
                next_url = api.content.get(UID=uid).absolute_url()

                # update uids in session
                index = new_uids.index(uid)
                previous_uids = list(reversed(
                    get_previous_items(new_uids, index)))
                next_uids = get_next_items(new_uids, index)
                session[PREVIOUS_UIDS] = json.dumps(previous_uids)
                session[NEXT_UIDS] = json.dumps(next_uids)

                request.response.redirect(next_url)
                return  # don't expire session data

        if session.has_key(SEARCH_URL):  # noqa
            request.response.redirect(session[SEARCH_URL])
        else:
            request.response.redirect(api.portal.get().absolute_url())

        expire_session_data(request)
        return


class GoToPreviousItem(GoToNextItem):

    """Redirect to previous item in query."""

    uids_param = PREVIOUS_UIDS

    def get_uids(self):
        """Reverse uids."""
        uids = super(GoToPreviousItem, self).get_uids()
        uids.reverse()
        return uids



# -*- coding: utf-8 -*-
"""Subscribers."""
import json

from DateTime import DateTime
from zope.component import getAdapters
from zope.globalrequest import getRequest

from collective.querynextprev import QUERY, SEARCH_URL
from collective.querynextprev.interfaces import IAdditionalDataProvider
from collective.querynextprev.utils import clean_query


def convert_dates(obj):
    """Convert dates to iso format for json serialization."""
    if isinstance(obj, DateTime):
        return obj.ISO()
    else:
        raise TypeError(repr(obj) + " is not JSON serializable")


def record_query_in_session(obj, event):
    """Record catalog query in session."""
    request = getRequest()
    session = request.SESSION
    session[QUERY] = json.dumps(clean_query(event.query), default=convert_dates)
    session[SEARCH_URL] = request.HTTP_REFERER
    adapters = getAdapters((obj, ), IAdditionalDataProvider)
    for adapter in dict(adapters).values():
        session[adapter.get_key()] = adapter.get_value()



# -*- coding: utf-8 -*-
"""Utils."""
import collections


WINDOW_SIZE = 10


def expire_session_data(request):
    """Expire all querynextprev data in session."""
    for key in request.SESSION.keys():
        if key.startswith('querynextprev'):
            del request.SESSION[key]


def first_common_item(l1, l2):
    """Get first item in l2 that is also in l1."""
    for item in l2:
        if item in l1:
            return item

    return None


def get_next_items(l, index, include_index=False):
    """Get WINDOW_SIZE next items."""
    last_index = min(index + WINDOW_SIZE, len(l))
    if include_index:
        index -= 1

    return l[index+1:last_index+1]


def get_previous_items(l, index, include_index=False):
    """Get WINDOW_SIZE previous items."""
    first_index = max(index - 10, 0)
    if include_index:
        index += 1

    return l[first_index:index]


def convert_to_str(value):
    """Converts a value to str."""
    # pylint: disable=W0141
    if isinstance(value, str):
        return value
    if isinstance(value, unicode):
        return value.encode('utf8')
    elif isinstance(value, collections.Mapping):
        return dict(map(convert_to_str, value.iteritems()))
    elif isinstance(value, collections.Iterable):
        return type(value)(map(convert_to_str, value))
    else:
        return value


def clean_query(query):
    """ Remove from eeafacetednavigation query useless keys """
    return {k: v for k, v in query.items() if k not in ('facet.field', 'b_size', 'b_start')}



# -*- coding: utf-8 -*-
import logging
from plone import api

logger = logging.getLogger('collective.querynextprev: upgrade. ')


def v1001(context):
    setup = api.portal.get_tool('portal_setup')
    setup.runImportStepFromProfile('profile-collective.querynextprev:default', 'plone.app.registry')



# -*- coding: utf-8 -*-
import json
import unittest2 as unittest

from plone import api
from plone.app.testing import login, setRoles, TEST_USER_ID, TEST_USER_NAME

from collective.querynextprev import QUERY, SEARCH_URL, PREVIOUS_UIDS, NEXT_UIDS  # noqa #pylint: disable=C0301
from collective.querynextprev.browser.viewlets import NextPrevNavigationViewlet
from collective.querynextprev.tests import query, DummyView
from collective.querynextprev.testing import COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING  # noqa #pylint: disable=C0301


class TestNextPrevNavigationViewlet(unittest.TestCase):

    """Test NextPrevNavigationViewlet."""

    layer = COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING

    def setUp(self):
        portal = api.portal.get()
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        self.doc = api.content.create(
            id='mydoc', type='Document', container=portal)
        self.view = DummyView()
        portal.REQUEST.SESSION = {}
        self.portal = portal

    def test_no_query_set(self):
        portal = self.portal
        request = portal.REQUEST
        viewlet = NextPrevNavigationViewlet(self.doc, request, self.view)
        viewlet.update()
        session = request.SESSION
        for key in [QUERY, SEARCH_URL, PREVIOUS_UIDS, NEXT_UIDS]:
            self.assertNotIn(key, session)

        self.assertFalse(viewlet.is_navigable)

    def test_alone(self):
        portal = self.portal
        request = portal.REQUEST
        session = request.SESSION
        session[QUERY] = query
        viewlet = NextPrevNavigationViewlet(self.doc, request, self.view)
        viewlet.update()
        self.assertNotIn(QUERY, session)
        self.assertNotIn(PREVIOUS_UIDS, session)
        self.assertNotIn(NEXT_UIDS, session)
        self.assertFalse(viewlet.is_navigable)

    def test_maxresults(self):
        portal = self.portal
        request = portal.REQUEST
        session = request.SESSION
        api.content.create(id='mydoc2', type='Document', container=portal)
        session[QUERY] = query
        viewlet = NextPrevNavigationViewlet(self.doc, request, self.view)
        viewlet.update()
        self.assertIn(QUERY, session)
        self.assertIn(PREVIOUS_UIDS, session)
        self.assertIn(NEXT_UIDS, session)
        self.assertTrue(viewlet.is_navigable)
        # we set maxresults to 1
        api.portal.set_registry_record(name='collective.querynextprev.maxresults', value=1)
        self.assertIn(QUERY, session)
        viewlet.update()
        self.assertNotIn(QUERY, session)
        self.assertNotIn(PREVIOUS_UIDS, session)
        self.assertNotIn(NEXT_UIDS, session)
        self.assertFalse(viewlet.is_navigable)

    def test_one_after(self):
        """Test when there is a next item and no previous item."""
        portal = self.portal
        request = portal.REQUEST
        session = request.SESSION
        session[QUERY] = query
        doc1 = self.doc
        doc2 = api.content.create(
            id='mydoc2', type='Document', container=portal)
        viewlet = NextPrevNavigationViewlet(doc1, request, self.view)
        viewlet.update()
        self.assertEqual(session[QUERY], query)
        previous_uids = json.loads(session[PREVIOUS_UIDS])
        next_uids = json.loads(session[NEXT_UIDS])
        self.assertNotIn(doc1.UID(), previous_uids)
        self.assertNotIn(doc1.UID(), next_uids)
        self.assertNotIn(doc2.UID(), previous_uids)
        self.assertIn(doc2.UID(), next_uids)
        self.assertTrue(viewlet.is_navigable)

    def test_one_before(self):
        """Test when there is a previous item and no next item."""
        portal = self.portal
        request = portal.REQUEST
        session = request.SESSION
        session[QUERY] = query
        doc1 = self.doc
        doc2 = api.content.create(
            id='mydoc2', type='Document', container=portal)
        viewlet = NextPrevNavigationViewlet(doc2, request, self.view)
        viewlet.update()
        self.assertEqual(session[QUERY], query)
        previous_uids = json.loads(session[PREVIOUS_UIDS])
        next_uids = json.loads(session[NEXT_UIDS])
        self.assertIn(doc1.UID(), previous_uids)
        self.assertNotIn(doc1.UID(), next_uids)
        self.assertNotIn(doc2.UID(), previous_uids)
        self.assertNotIn(doc2.UID(), next_uids)
        self.assertTrue(viewlet.is_navigable)

    def test_window(self):
        """Test that 10 items before and 10 items after are kept in session."""
        portal = self.portal
        request = portal.REQUEST
        session = request.SESSION
        session[QUERY] = query
        for x in range(100):
            name = "mydoc-{}".format(x)
            api.content.create(id=name, type='Document', container=portal)

        viewlet = NextPrevNavigationViewlet(self.doc, request, self.view)
        viewlet.update()
        self.assertEqual(session[QUERY], query)
        previous_uids = json.loads(session[PREVIOUS_UIDS])
        next_uids = json.loads(session[NEXT_UIDS])
        self.assertEqual(len(previous_uids), 0)
        self.assertEqual(len(next_uids), 10)
        next_ = [api.content.get(UID=uid).getId() for uid in next_uids]
        self.assertEqual(
            next_,
            ['mydoc-0', 'mydoc-1', 'mydoc-2', 'mydoc-3', 'mydoc-4',
             'mydoc-5', 'mydoc-6', 'mydoc-7', 'mydoc-8', 'mydoc-9']
            )
        self.assertTrue(viewlet.is_navigable)

        doc = portal['mydoc-50']
        viewlet = NextPrevNavigationViewlet(doc, request, self.view)
        viewlet.update()
        self.assertEqual(session[QUERY], query)
        previous_uids = json.loads(session[PREVIOUS_UIDS])
        next_uids = json.loads(session[NEXT_UIDS])
        self.assertEqual(len(previous_uids), 10)
        self.assertEqual(len(next_uids), 10)
        self.assertTrue(viewlet.is_navigable)

        previous = [api.content.get(UID=uid).getId() for uid in previous_uids]
        self.assertEqual(
            previous,
            ['mydoc-49', 'mydoc-48', 'mydoc-47', 'mydoc-46', 'mydoc-45',
             'mydoc-44', 'mydoc-43', 'mydoc-42', 'mydoc-41', 'mydoc-40']
            )

        next_ = [api.content.get(UID=uid).getId() for uid in next_uids]
        self.assertEqual(
            next_,
            ['mydoc-51', 'mydoc-52', 'mydoc-53', 'mydoc-54', 'mydoc-55',
             'mydoc-56', 'mydoc-57', 'mydoc-58', 'mydoc-59', 'mydoc-60']
            )

    def test_context_not_in_results(self):
        """Test the case in which context is not in the results."""
        portal = self.portal
        request = portal.REQUEST
        query = json.dumps({
            'portal_type': 'Document',
            'sort_on': 'sortable_title',
            'SearchableText': 'Great title'
            })
        session = request.SESSION
        session[QUERY] = query
        docs = []
        for x in range(10):
            name = "mydoc-{}".format(x)
            doc = api.content.create(
                id=name, title="Great title",
                type='Document', container=portal)
            docs.append(doc)

        old_previous_uids = list(reversed([doc.UID() for doc in docs[:5]]))  # noqa #pylint: disable=C0301
        session[PREVIOUS_UIDS] = json.dumps(old_previous_uids)
        old_next_uids = [doc.UID() for doc in docs[8:]]
        session[NEXT_UIDS] = json.dumps(old_next_uids)
        viewlet = NextPrevNavigationViewlet(self.doc, request, self.view)
        viewlet.update()
        self.assertEqual(session[QUERY], query)
        self.assertTrue(viewlet.is_navigable)
        self.assertEqual(
            json.loads(session[PREVIOUS_UIDS]), old_previous_uids)
        self.assertEqual(
            json.loads(session[NEXT_UIDS]), old_next_uids)
        self.assertEqual(viewlet.previous_uids, old_previous_uids)
        self.assertEqual(viewlet.next_uids, old_next_uids)



# -*- coding: utf-8 -*-
"""Test subscribers."""
import unittest2 as unittest

from plone import api

from collective.querynextprev.subscribers import record_query_in_session
from collective.querynextprev.testing import COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING  # noqa #pylint: disable=C0301
from collective.querynextprev import QUERY, SEARCH_URL


class DummyEvent(object):
    pass


class TestSubscribers(unittest.TestCase):

    """Test subscribers."""

    layer = COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING

    def test_record_query_in_session(self):
        """Test record_query_in_session subscriber."""
        portal = api.portal.get()
        request = portal.REQUEST
        request.SESSION = {}
        event = DummyEvent()
        event.query = {'k': 'foobar'}
        record_query_in_session(portal, event)
        self.assertIn(QUERY, request.SESSION)
        self.assertIn(SEARCH_URL, request.SESSION)



# -*- coding: utf-8 -*-
"""Init and utils."""

from zope.i18nmessageid import MessageFactory

_ = MessageFactory('collective.querynextprev')

QUERY = 'querynextprev.query'
SEARCH_URL = 'querynextprev.search_url'
PREVIOUS_UIDS = 'querynextprev.previous_uids'
NEXT_UIDS = 'querynextprev.next_uids'



# -*- coding: utf-8 -*-
"""Test utilities."""
import unittest2 as unittest

from plone import api

from collective.querynextprev.utils import (
    expire_session_data, first_common_item, get_next_items, get_previous_items, clean_query)
from collective.querynextprev.testing import COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING  # noqa #pylint: disable=C0301


class TestUtils(unittest.TestCase):

    """Test NextPrevNavigationViewlet."""

    layer = COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING

    def setUp(self):
        portal = api.portal.get()
        self.request = portal.REQUEST

    def tearDown(self):
        if hasattr(self.request, 'SESSION'):
            del self.request.SESSION

    def test_expire_session_data(self):
        """Test expire_session_data function."""
        request = self.request
        request.SESSION = {}
        expire_session_data(request)
        self.assertEqual(request.SESSION, {})

        request.SESSION = {
            'foo': 'bar',
            'querynextprev.foo': 'bar',
            'querynextprev.bar': 'foo',
        }
        expire_session_data(request)
        self.assertEqual(request.SESSION, {'foo': 'bar'})

    def test_first_common_item(self):
        """Test first common item util."""
        l1 = [4, 5, 6, 7]
        l2 = [1, 2, 6, 7]

        self.assertEqual(
            first_common_item(l1, l2),
            6)

        l1 = [1, 2, 4, 5, 6, 7]
        l2 = [1, 2, 6, 7]
        self.assertEqual(
            first_common_item(l1, l2),
            1)

        l1 = [4, 5, 6, 7]
        l2 = [1, 2]
        self.assertIsNone(
            first_common_item(l1, l2))

        l1 = [1]
        l2 = [1]
        self.assertEqual(
            first_common_item(l1, l2),
            1)

    def test_get_next_items(self):
        """Test get_next_items function."""
        l = range(40)
        index = 19
        self.assertEqual(
            get_next_items(l, index),
            range(20, 30)
        )

        index = 35
        self.assertEqual(
            get_next_items(l, index),
            range(36, 40)
        )

        self.assertEqual(
            get_next_items(l, index, include_index=True),
            range(35, 40)
        )

    def test_get_previous_items(self):
        """Test get_previous_items function."""
        l = range(40)
        index = 21
        self.assertEqual(
            get_previous_items(l, index),
            range(11, 21)
        )

        index = 5
        self.assertEqual(
            get_previous_items(l, index),
            range(5)
        )

        self.assertEqual(
            get_previous_items(l, index, include_index=True),
            range(6)
        )

    def test_clean_query(self):
        query = {'sort_order': 'descending', 'Language': ['fr', ''], 'sort_on': 'created',
                 'facet.field': ['', u'review_state', u'treating_groups', u'assigned_user', u'recipient_groups',
                                 u'mail_type'],
                 'b_size': 24, 'b_start': 0, 'portal_type': {'query': ['dmsincomingmail']}}
        self.assertDictEqual(clean_query(query), {'sort_order': 'descending', 'Language': ['fr', ''],
                                                  'sort_on': 'created', 'portal_type': {'query': ['dmsincomingmail']}})



# -*- coding: utf-8 -*-
import json


query = json.dumps({
    'portal_type': 'Document',
    'sort_on': 'sortable_title'
})

query_utf8 = json.dumps({
    'portal_type': 'Document',
    'sort_on': 'id',
    'Title': 'é'
})


class DummyView(object):
    pass



# -*- coding: utf-8 -*-


def post_install(context):
    """Post install script"""
    if context.readDataFile('collectivequerynextprev_default.txt') is None:
        return  # pragma: no cover
    # Do something during the installation of this package



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from collective.querynextprev.testing import COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING  # noqa
from plone import api

import unittest2 as unittest


class TestSetup(unittest.TestCase):
    """Test that collective.querynextprev is properly installed."""

    layer = COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if collective.querynextprev is installed with portal_quickinstaller."""  # noqa
        self.assertTrue(
            self.installer.isProductInstalled('collective.querynextprev'))

    def test_browserlayer(self):
        """Test that ICollectiveQuerynextprevLayer is registered."""
        from collective.querynextprev.interfaces import ICollectiveQuerynextprevLayer  # noqa
        from plone.browserlayer import utils
        self.assertIn(ICollectiveQuerynextprevLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):

    layer = COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.uninstallProducts(['collective.querynextprev'])

    def test_product_uninstalled(self):
        """Test if collective.querynextprev is cleanly uninstalled."""
        self.assertFalse(
            self.installer.isProductInstalled('collective.querynextprev'))



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""

from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.interface import Interface
from zope.viewlet.interfaces import IViewletManager


class ICollectiveQuerynextprevLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class INextPrevViewletManager(IViewletManager):

    """Viewlet manager for next/previous functionnality."""


class IAdditionalDataProvider(Interface):

    """Additional data provider."""

    def get_key(self):
        """Key under which the data will be stored in session."""

    def get_value(self):
        """Data that will be stored in session."""


class INextPrevNotNavigable(Interface):

    """Marker interface for contents that are not next/prev navigables."""



# -*- coding: utf-8 -*-
"""Test views."""
import json
import unittest2 as unittest

from plone import api
from plone.app.testing import login, setRoles, TEST_USER_ID, TEST_USER_NAME

from collective.querynextprev import QUERY, SEARCH_URL, NEXT_UIDS, PREVIOUS_UIDS  # noqa #pylint: disable=C0301
from collective.querynextprev.browser.views import GoToNextItem, GoToPreviousItem  # noqa #pylint: disable=C0301
from collective.querynextprev.tests import query_utf8
from collective.querynextprev.testing import COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING  # noqa #pylint: disable=C0301


class TestGoToNextItem(unittest.TestCase):

    """Test GoToNextItem view."""

    layer = COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING

    def setUp(self):
        portal = api.portal.get()
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        for x in range(30):
            name = "mydoc-{:02d}".format(x + 1)
            api.content.create(id=name, type='Document', container=portal, title='é')

        self.doc1 = portal['mydoc-01']
        self.doc2 = portal['mydoc-02']
        self.doc3 = portal['mydoc-03']
        self.doc11 = portal['mydoc-11']
        self.doc30 = portal['mydoc-30']
        self.portal = portal

    def test_get_uids(self):
        """Test get_uids method."""
        request = self.portal.REQUEST
        request.SESSION = {
            QUERY: query_utf8,
        }
        context = self.portal
        view = GoToNextItem(context, request)
        uids = view.get_uids()
        self.assertEqual(len(uids), 30)
        self.assertEqual(uids[0], self.doc1.UID())
        self.assertEqual(uids[10], self.doc11.UID())
        self.assertEqual(uids[29], self.doc30.UID())

    def test_view(self):
        doc1 = self.doc1
        doc2 = self.doc2
        doc3 = self.doc3
        portal = self.portal
        request = portal.REQUEST

        # no query, no search url
        request.SESSION = {}
        view = GoToNextItem(doc1, request)
        view()
        self.assertEqual(
            request.response.getHeader('location'),
            portal.absolute_url()
            )

        # no search url
        request.SESSION = {
            SEARCH_URL: 'http://www.example.com'
            }
        view = GoToNextItem(doc1, request)
        view()
        self.assertEqual(
            request.response.getHeader('location'),
            'http://www.example.com'
            )

        # with a query
        request.SESSION = {
            QUERY: query_utf8,
            PREVIOUS_UIDS: json.dumps([]),
            NEXT_UIDS: json.dumps([doc2.UID(), doc3.UID()]),
            SEARCH_URL: 'http://www.example.com'
            }
        view = GoToNextItem(doc1, request)
        view()
        self.assertEqual(
            request.response.getHeader('location'),
            doc2.absolute_url()
            )

        # with a query, first next item deleted
        request.SESSION = {
            QUERY: query_utf8,
            PREVIOUS_UIDS: json.dumps([]),
            NEXT_UIDS: json.dumps([doc2.UID(), doc3.UID()]),
            SEARCH_URL: 'http://www.example.com'
            }
        view = GoToNextItem(doc1, request)
        api.content.delete(doc2)
        view()
        self.assertEqual(
            request.response.getHeader('location'),
            doc3.absolute_url()
            )


class TestGoToPreviousItem(unittest.TestCase):

    """Test GoToPreviousItem view."""

    layer = COLLECTIVE_QUERYNEXTPREV_INTEGRATION_TESTING

    def setUp(self):
        portal = api.portal.get()
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        for x in range(30):
            name = "mydoc-{:02d}".format(x + 1)
            api.content.create(id=name, type='Document', container=portal, title='é')

        self.doc1 = portal['mydoc-01']
        self.doc2 = portal['mydoc-02']
        self.doc3 = portal['mydoc-03']
        self.doc20 = portal['mydoc-20']
        self.doc30 = portal['mydoc-30']
        self.portal = portal

    def test_get_uids(self):
        """Test get_uids method."""
        request = self.portal.REQUEST
        request.SESSION = {
            QUERY: query_utf8,
        }
        context = self.portal
        view = GoToPreviousItem(context, request)
        uids = view.get_uids()
        self.assertEqual(len(uids), 30)
        self.assertEqual(uids[0], self.doc30.UID())
        self.assertEqual(uids[10], self.doc20.UID())
        self.assertEqual(uids[29], self.doc1.UID())

    def test_view(self):
        doc1 = self.doc1
        doc2 = self.doc2
        doc3 = self.doc3
        portal = self.portal
        request = portal.REQUEST

        # with a query
        request.SESSION = {
            QUERY: query_utf8,
            PREVIOUS_UIDS: json.dumps([doc2.UID(), doc1.UID()]),
            NEXT_UIDS: json.dumps([]),
            SEARCH_URL: 'http://www.example.com'
            }
        view = GoToPreviousItem(doc3, request)
        view()
        self.assertEqual(
            request.response.getHeader('location'),
            doc2.absolute_url()
            )

        # with a query, first next item deleted
        request.SESSION = {
            QUERY: query_utf8,
            PREVIOUS_UIDS: json.dumps([doc1.UID()]),
            NEXT_UIDS: json.dumps([]),
            SEARCH_URL: 'http://www.example.com'
            }
        api.content.delete(doc1)
        view = GoToPreviousItem(doc2, request)
        view()
        self.assertEqual(
            request.response.getHeader('location'),
            'http://www.example.com'
            )
